/*

   []----------------------------------------------------------------------[]
   |                               MOUSE.C                                  |
   |                                                                        |
   |                  Funcoes para manipulacao do mouse.                    |
   |                                                                        |
   |                  (C) 1995, Lenimar Nunes de Andrade                    |
   []----------------------------------------------------------------------[]

*/

#define MOUSE        0x33
#define BOTAO_ESQUERDO  0
#define BOTAO_DIREITO   1
#define BOTAO_CENTRAL   2
#define NAO_PRESSIONADO 3

/* Definicoes dos formatos dos indicadores do mouse em modo grafico */

unsigned int Arrow[] = {
 0xFFFF, 0xBFFF, 0x9FFF, 0x8FFF, 0x87FF, 0x83FF, 0x81FF, 0x80FF,
 0x807F, 0x83FF, 0xB3FF, 0xF3FF, 0xF9FF, 0xF9FF, 0xF9FF, 0xFFFF,
    0x0, 0x4000, 0x6000, 0x7000, 0x7800, 0x7C00, 0x7E00, 0x7F00,
 0x7F80, 0x7C00, 0x4C00,  0xC00,  0x600,  0x600,  0x600,    0x0
};

unsigned int Ibeam[] = {
 0xF3CF, 0xFDBF, 0xFE7F, 0xFE7F, 0xFE7F, 0xFE7F, 0xFE7F, 0xFE7F,
 0xFE7F, 0xFE7F, 0xFE7F, 0xFE7F, 0xFE7F, 0xFE7F, 0xFDBF, 0xF3CF,
  0xC30,  0x240,  0x180,  0x180,  0x180,  0x180,  0x180,  0x180,
  0x180,  0x180,  0x180,  0x180,  0x180,  0x180,  0x240,  0xC30
};

unsigned int Glove[] = {
 0xF3FF, 0xEDFF, 0xEDFF, 0xEDFF, 0xEDFF, 0xEC49, 0xEDB6, 0x8DB6,
 0x6DB6, 0x6FFE, 0x6FFE, 0x7FFE, 0x7FFE, 0xBFFD, 0xBFFD, 0xC003,
  0xC00, 0x1200, 0x1200, 0x1200, 0x1200, 0x13B6, 0x1249, 0x7249,
 0x9249, 0x9001, 0x9001, 0x8001, 0x8001, 0x4002, 0x4002, 0x3FFC
};

unsigned int Cross[] = {
 0xFFFF, 0xF80F, 0xF777, 0xEF7B, 0xDF7D, 0xBF7E, 0xBF7E, 0xBFFE,
 0x81C0, 0xBFFE, 0xBF7E, 0xBF7E, 0xDF7D, 0xEF7B, 0xF777, 0xF80F,
    0x0,  0x7F0,  0x888, 0x1084, 0x2082, 0x4081, 0x4081, 0x4001,
 0x7E3F, 0x4001, 0x4081, 0x4081, 0x2082, 0x1084,  0x888,  0x7F0
};

unsigned int Check[] = {
 0xFFFF, 0xFFF9, 0xFFF3, 0xFFE7, 0xFFCF, 0xFF9F, 0x8F3F, 0xC67F,
 0xE0FF, 0xF1FF, 0xFBFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF,
    0x0,    0x6,    0xC,   0x18,   0x30,   0x60, 0x70C0, 0x3980,
 0x1F00,  0xE00,  0x400,    0x0,    0x0,    0x0,    0x0,    0x0
};

#define X_GLOVE  5     /* Definicoes dos pontos fortes */
#define Y_GLOVE  1     /* dos indicadores do mouse     */
#define X_CHECK  6
#define Y_CHECK 11
#define X_CROSS  8
#define Y_CROSS  8
#define X_IBEAM  8
#define Y_IBEAM  8
#define X_ARROW  2
#define Y_ARROW  2

/* ------------------------------------------------------------------------ */

int HaSuporteAoMouse(void) {

  union REGS regs;

  regs.x.ax = 0;
  int86(MOUSE, &regs, &regs);

  if (regs.x.ax == 0xFFFF)
    return regs.x.bx; /* Ha' suporte ao mouse. Retorna a
			 quantidade de botoes do  mouse. */
  else
    return 0;         /* Nao ha' suporte ao mouse */
}

/* ------------------------------------------------------------------------ */

void MostraIndicadorDoMouse(void) {

  union REGS regs;

  regs.x.ax = 1;
  int86(MOUSE, &regs, &regs);
}

/* ------------------------------------------------------------------------ */

void EscondeIndicadorDoMouse(void) {

  union REGS regs;

  regs.x.ax = 2;
  int86(MOUSE, &regs, &regs);
}

/* ------------------------------------------------------------------------ */

int ObtemPosicaoDoIndicadorDoMouse(int *pos_x, int *pos_y) {

/* As coordenadas sao devolvidas em pixels independentemente do modo de
   apresentacao atual. A posicao (0, 0) e' o canto superior esquerdo da
   tela. */

  union REGS regs;

  regs.x.ax = 3;
  int86(MOUSE, &regs, &regs);
  *pos_x = regs.x.cx;
  *pos_y = regs.x.dx;
  if (regs.x.bx & 1) return BOTAO_ESQUERDO;   /* botao esquerdo pressionado */
  else if (regs.x.bx &2) return BOTAO_DIREITO; /* botao direito pressionado */
  else if (regs.x.bx &4) return BOTAO_CENTRAL; /* botao central pressionado */
  else return NAO_PRESSIONADO;                 /* nenhum  botao pressionado */
}

/* ------------------------------------------------------------------------ */

void AjustaPosicaoDoIndicadorDoMouse(int pos_x, int pos_y) {

/* As coordenadas sao devolvidas em pixels independentemente do modo de
   apresentacao atual. A posicao (0, 0) e' o canto superior esquerdo da
   tela.
   A posicao e' ajustada se necessario para permanecer dentro dos limi-
   tes horizontal e vertical especificados. */

  union REGS regs;

  regs.x.ax = 4;
  regs.x.cx = pos_x;
  regs.x.dx = pos_y;
  int86(MOUSE, &regs, &regs);
}

/* ------------------------------------------------------------------------ */

void AjustaLimitesVerticais(int liminf, int limsup) {

  union REGS regs;

  regs.x.ax = 8;
  regs.x.cx = liminf;
  regs.x.dx = limsup;
  int86(MOUSE, &regs, &regs);
}

/* ------------------------------------------------------------------------ */

void AjustaLimitesHorizontais(int liminf, int limsup) {

  union REGS regs;

  regs.x.ax = 7;
  regs.x.cx = liminf;
  regs.x.dx = limsup;
  int86(MOUSE, &regs, &regs);
}

/* ------------------------------------------------------------------------ */

int BotaoPressionado(int *pos_x, int *pos_y) {

  union REGS regs;
  int botao = BOTAO_ESQUERDO;

  regs.x.ax = 5;
  regs.x.bx = botao;
  int86(MOUSE, &regs, &regs);

  *pos_x = regs.x.cx;
  *pos_y = regs.x.dx;

  if (regs.x.ax) return 1; else return 0;

}

/* ------------------------------------------------------------------------ */

int VerificaPosIndicadorDoMouse(int x_min, int y_min, int x_max, int y_max) {

/*  Verifica se o indicador do mouse se encontra no retangulo definido
    pelas coordenadas fornecidas como parametros.

		  (x_min, y_min)
			+------------------------+
			|              X         |
			|                        |
			+------------------------+
					   (x_max, y_max) */

  union REGS regs;
  int pos_x, pos_y;

  regs.x.ax = 3;
  int86(MOUSE, &regs, &regs);
  pos_x = regs.x.cx;
  pos_y = regs.x.dx;

  return (x_min <= pos_x && pos_x <= x_max &&
	  y_min <= pos_y && pos_y <= y_max);
}

/* ------------------------------------------------------------------------ */

void AjustaFormaDoIndicadorGrafico(int desl_esquerda, int desl_topo,
				   long formato) {

/* Define forma, cor e ponto forte do indicador do mouse em modo grafico.
   desl_esquerda e' o deslocamento do ponto forte a partir  da  esquerda,
   desl_topo  e'  o  deslocamento  do  ponto  forte  a  partir  do  topo,
   segmento:deslocamento e' o endereco do inicio  da matriz de imagem  do
			 indicador. */

  struct REGPACK regs;

  regs.r_ax = 9;
  regs.r_bx = desl_esquerda;
  regs.r_cx = desl_topo;
  regs.r_es = formato >> 16;
  regs.r_dx = formato & 0xFFFF;
  intr(MOUSE, &regs);
}

/* ------------------------------------------------------------------------ */

/*** FIM DE "MOUSE.C" ***/
